torch_version_list = [
    {
        "version": "2.4.0",
        "install": "pip install torch==2.4.0 torchvision==0.19.0 torchaudio==2.4.0 --index-url https://download.pytorch.org/whl/cu118 --extra-index-url https://pypi.tuna.tsinghua.edu.cn/simple",
        "image": "nvidia/cuda:11.8.0-cudnn8-devel-ubuntu22.04",
    },
    {
        "version": "2.3.1",
        "install": "pip install torch==2.3.1 torchvision==0.18.1 torchaudio==2.3.1 --index-url https://download.pytorch.org/whl/cu118 --extra-index-url https://pypi.tuna.tsinghua.edu.cn/simple",
        "image": "nvidia/cuda:11.8.0-cudnn8-devel-ubuntu22.04",
    },
    {
        "version": "2.3.0",
        "install": "pip install torch==2.3.0 torchvision==0.18.0 torchaudio==2.3.0 --index-url https://download.pytorch.org/whl/cu118 --extra-index-url https://pypi.tuna.tsinghua.edu.cn/simple",
        "image": "nvidia/cuda:11.8.0-cudnn8-devel-ubuntu22.04",
    },
    {
        "version": "2.2.2",
        "install": "pip install torch==2.2.2 torchvision==0.17.2 torchaudio==2.2.2 --index-url https://download.pytorch.org/whl/cu118 --extra-index-url https://pypi.tuna.tsinghua.edu.cn/simple",
        "image": "nvidia/cuda:11.8.0-cudnn8-devel-ubuntu22.04",
    },
    {
        "version": "2.2.1",
        "install": "pip install torch==2.2.1 torchvision==0.17.1 torchaudio==2.2.1 --index-url https://download.pytorch.org/whl/cu118 --extra-index-url https://pypi.tuna.tsinghua.edu.cn/simple",
        "image": "nvidia/cuda:11.8.0-cudnn8-devel-ubuntu22.04",
    },
    {
        "version": "2.2.0",
        "install": "pip install torch==2.2.0 torchvision==0.17.0 torchaudio==2.2.0 --index-url https://download.pytorch.org/whl/cu118 --extra-index-url https://pypi.tuna.tsinghua.edu.cn/simple",
        "image": "nvidia/cuda:11.8.0-cudnn8-devel-ubuntu22.04",
    },
    {
        "version": "2.1.2",
        "install": "pip install torch==2.1.2 torchvision==0.16.2 torchaudio==2.1.2 --index-url https://download.pytorch.org/whl/cu118 --extra-index-url https://pypi.tuna.tsinghua.edu.cn/simple",
        "image": "nvidia/cuda:11.8.0-cudnn8-devel-ubuntu22.04",
    },
    {
        "version": "2.1.1",
        "install": "pip install torch==2.1.1 torchvision==0.16.1 torchaudio==2.1.1 --index-url https://download.pytorch.org/whl/cu118 --extra-index-url https://pypi.tuna.tsinghua.edu.cn/simple",
        "image": "nvidia/cuda:11.8.0-cudnn8-devel-ubuntu22.04",
    },
    {
        "version": "2.1.0",
        "install": "pip install torch==2.1.0 torchvision==0.16.0 torchaudio==2.1.0 --index-url https://download.pytorch.org/whl/cu118 --extra-index-url https://pypi.tuna.tsinghua.edu.cn/simple",
        "image": "nvidia/cuda:11.8.0-cudnn8-devel-ubuntu22.04",
    },
    {
        "version": "2.0.1",
        "install": "pip install torch==2.0.1 torchvision==0.15.2 torchaudio==2.0.2 --index-url https://download.pytorch.org/whl/cu118 --extra-index-url https://pypi.tuna.tsinghua.edu.cn/simple",
        "image": "nvidia/cuda:11.8.0-cudnn8-devel-ubuntu22.04",
    },
    {
        "version": "2.0.0",
        "install": "pip install torch==2.0.0 torchvision==0.15.1 torchaudio==2.0.1 --index-url https://download.pytorch.org/whl/cu118 --extra-index-url https://pypi.tuna.tsinghua.edu.cn/simple",
        "image": "nvidia/cuda:11.8.0-cudnn8-devel-ubuntu22.04",
    },
    {
        "version": "1.13.1",
        "install": "pip install torch==1.13.1+cu117 torchvision==0.14.1+cu117 torchaudio==0.13.1 --extra-index-url https://download.pytorch.org/whl/cu117 --index-url https://pypi.tuna.tsinghua.edu.cn/simple",
        "image": "nvidia/cuda:11.7.1-cudnn8-devel-ubuntu22.04",
    },
    {
        "version": "1.13.0",
        "install": "pip install torch==1.13.0+cu117 torchvision==0.14.0+cu117 torchaudio==0.13.0 --extra-index-url https://download.pytorch.org/whl/cu117 --index-url https://pypi.tuna.tsinghua.edu.cn/simple",
        "image": "nvidia/cuda:11.7.1-cudnn8-devel-ubuntu22.04",
    },
    {
        "version": "1.12.1",
        "install": "pip install torch==1.12.1+cu116 torchvision==0.13.1+cu116 torchaudio==0.12.1 --extra-index-url https://download.pytorch.org/whl/cu116 --index-url https://pypi.tuna.tsinghua.edu.cn/simple",
        "image": "nvidia/cuda:11.6.1-cudnn8-devel-ubuntu20.04",
    },
    {
        "version": "1.12.0",
        "install": "pip install torch==1.12.0+cu116 torchvision==0.13.0+cu116 torchaudio==0.12.0 --extra-index-url https://download.pytorch.org/whl/cu116 --index-url https://pypi.tuna.tsinghua.edu.cn/simple",
        "image": "nvidia/cuda:11.6.1-cudnn8-devel-ubuntu20.04",
    },
]
